<?php

namespace Drupal\access_token_auth\Authentication;

use Drupal\access_token_auth\Services\TokenManager;
use Drupal\Core\Authentication\AuthenticationProviderInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\StringTranslation\TranslationInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Access Token Auth authentication provider.
 */
class AccessTokenAuthProvider implements AuthenticationProviderInterface {

  const TOKEN = 'X-ACCESS-AUTH-TOKEN';

  /**
   * The token manager service.
   *
   * @var \Drupal\access_token_auth\Service\TokenManager
   */
  protected TokenManager $tokenManager;

  /**
   * The translation service.
   *
   * @var \Drupal\Core\StringTranslation\TranslationInterface
   */
  protected TranslationInterface $translator;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * AccessTokenAuthProvider class constructor.
   *
   * @param \Drupal\access_token_auth\Service\TokenManager $token_manager
   *   The token manager service.
   * @param \Drupal\Core\StringTranslation\TranslationInterface $translator
   *   Translator.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(
    TokenManager $token_manager,
    TranslationInterface $translator,
    EntityTypeManagerInterface $entity_type_manager
  ) {
    $this->tokenManager = $token_manager;
    $this->translator = $translator;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public function applies(Request $request): bool {
    return !(empty($request->headers->get(self::TOKEN)) && empty($request->query->get(self::TOKEN)));
  }

  /**
   * {@inheritdoc}
   */
  public function authenticate(Request $request): ?AccountInterface {
    $token = (string) $request->headers->get(self::TOKEN);
    if (empty($token)) {
      $token = (string) $request->query->get(self::TOKEN);
    }

    if (empty($token)) {
      return NULL;
    }

    $token = $this->tokenManager->getToken($token);

    if (!$token) {
      return NULL;
    }

    // Invalidate token (i.e. mark it as used) regardless of auth mode.
    $this->tokenManager->invalidateToken($token->getToken());

    /** @var \Drupal\user\UserInterface|null */
    $user = $this->entityTypeManager->getStorage('user')->load($token->getUserId());

    if (!$user || !$user->isActive()) {
      return NULL;
    }

    return $user;
  }

}
