<?php

namespace Drupal\access_token_auth\Form;

use Drupal\access_token_auth\Model\Token;
use Drupal\access_token_auth\TokenGeneratorInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Psr\Container\ContainerInterface;

/**
 * Configure the token settings for this site.
 */
class TokenSettingsForm extends ConfigFormBase {

  /**
   * The token generator service.
   *
   * @var \Drupal\access_token_auth\TokenGeneratorInterface
   */
  protected TokenGeneratorInterface $tokenGenerator;

  /**
   * TokenSettingsForm class constructor.
   *
   * @param \Drupal\access_token_auth\TokenGeneratorInterface $token_generator
   *   The token generator service.
   */
  public function __construct(TokenGeneratorInterface $token_generator) {
    $this->tokenGenerator = $token_generator;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('access_token_auth.token_generator'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'access_token_auth_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['access_token_auth.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['mode'] = [
      '#type' => 'container',
      '#title' => $this->t('Authentication Mode'),
    ];

    $form['mode']['mode_options'] = [
      '#type' => 'radios',
      '#title' => $this->t('Mode'),
      '#description' => $this->t('Choose the token authencation mode'),
      '#required' => TRUE,
      '#options' => [
        TOKEN::DEFAULT_MODE => $this->t('Time based'),
        'one_time' => $this->t('One time'),
      ],
      '#default_value' => $this->config('access_token_auth.settings')->get('mode') ?? TOKEN::DEFAULT_MODE,
    ];

    $form['mode']['ttl'] = [
      '#type' => 'number',
      '#title' => $this->t('Token TTL'),
      '#description' => $this->t('The token time to live in seconds.'),
      '#default_value' => $this->config('access_token_auth.settings')->get('ttl') ?? Token::DEFAULT_TTL,
      '#min' => 30,
      '#max' => 1800,
      '#states' => [
        'visible' => [
          ':input[name="mode_options"]' => ['value' => 'time_based'],
        ],
      ],
    ];

    $form['generate_token'] = [
      '#type' => 'button',
      '#value' => $this->t('Generate Token'),
      '#ajax' => [
        'callback' => '::generateToken',
        'event' => 'click',
        'wrapper' => 'generated-token-output',
      ],
    ];

    $form['token_output'] = [
      '#markup' => "<div id='generated-token-output'></div>",
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    if ($form_state->getValue('mode_options') === 'time_based' && !$form_state->getValue('ttl')) {
      $form_state->setErrorByName('ttl', $this->t('You must set a value if the mode is time based.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('access_token_auth.settings')
      ->set('mode', $form_state->getValue('mode_options'))
      ->set('ttl', $form_state->getValue('ttl'))
      ->save();

    parent::submitForm($form, $form_state);
  }

  /**
   * Generate a token ajax callback.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  public function generateToken(array &$form, FormStateInterface $form_state) {
    $token = $this->tokenGenerator->generateToken($this->currentUser()->id());

    $token = $token?->getToken() ?? $this->t('There was an error trying to generate the token');
    $output = "<p id='generated-token-output'>$token</p>";

    return ['#markup' => $output];
  }

}
