<?php

namespace Drupal\access_token_auth\Services;

use Drupal\access_token_auth\Exception\TokenGeneratorException;
use Drupal\access_token_auth\Model\Token;
use Drupal\access_token_auth\TokenGeneratorInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Psr\Log\LoggerInterface;

/**
 * Generate unique tokens per user.
 */
class TokenGenerator implements TokenGeneratorInterface {

  /**
   * The token manager service.
   *
   * @var \Drupal\access_token_auth\Services\TokenManager
   */
  protected TokenManager $tokenManager;

  /**
   * The config factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected ConfigFactoryInterface $configFactory;

  /**
   * The logger service.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected LoggerInterface $logger;

  /**
   * TokenGenerator class constructor.
   *
   * @param \Drupal\access_token_auth\Services\TokenManager $token_manager
   *   The token manager service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory service.
   * @param \Psr\Log\LoggerInterface $logger
   *   The logger service.
   */
  public function __construct(
    TokenManager $token_manager,
    ConfigFactoryInterface $config_factory,
    LoggerInterface $logger
  ) {
    $this->tokenManager = $token_manager;
    $this->configFactory = $config_factory;
    $this->logger = $logger;
  }

  /**
   * {@inheritdoc}
   */
  public function generateToken(int $user_id): ?Token {
    try {
      if (!$token = $this->tokenManager->getValidUserToken($user_id)) {
        $token = $this->generateTransientToken($user_id);
        $this->tokenManager->insert($token);
        $this->logger->info('A token has been generated for user: @user_id', ['@user_id' => $user_id]);
      }

      return $token;
    }
    catch (\Throwable $exception) {
      $this->logger->error(
        'There was an error when trying to generate a token for user @user_id. \n Message: @message', [
          '@user_id' => $user_id,
          '@message' => $exception->getMessage(),
        ]
      );
    }

    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function generateTransientToken(int $user_id): Token {
    try {
      $token = hash('sha256', bin2hex(random_bytes(64)));
    }
    catch (\Throwable $exception) {
      throw new TokenGeneratorException($exception->getMessage());
    }

    if (empty($token)) {
      throw new TokenGeneratorException('No token was generated.');
    }

    $ttl = $this->configFactory->get('access_token_auth.settings')->get('ttl') ?: Token::DEFAULT_TTL;
    $expiration = time() + $ttl;

    return new Token($token, $user_id, $expiration);
  }

}
