<?php

namespace Drupal\access_token_auth\Services;

use Drupal\access_token_auth\Model\Token;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Database\Connection;

/**
 * Manager service for tokens.
 */
class TokenManager {

  /**
   * Database connection.
   *
   * @var \Drupal\Core\Database\Connection
   *   Database connection.
   */
  protected Connection $connection;

  /**
   * The config factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected ConfigFactoryInterface $configFactory;

  /**
   * TokenManager class constructor.
   *
   * @param \Drupal\Core\Database\Connection $connection
   *   Database connection.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory service.
   */
  public function __construct(
    Connection $connection,
    ConfigFactoryInterface $config_factory
  ) {
    $this->connection = $connection;
    $this->configFactory = $config_factory;
  }

  /**
   * Insert token into database.
   *
   * @param \Drupal\access_token_auth\Model\Token $token
   *   The token object.
   *
   * @return int
   *   The id in database after insert.
   *
   * @throws \Exception
   *   Database exception.
   */
  public function insert(Token $token): int {
    return (int) $this->connection
      ->insert(Token::TABLE_NAME)
      ->fields([
        'token' => $token->getToken(),
        'user_id' => $token->getUserId(),
        'expire_at' => $token->getExpireAt(),
      ])
      ->execute();
  }

  /**
   * Get token from the database.
   *
   * @param string $token
   *   The token passed in to load.
   *
   * @return \Drupal\access_token_auth\Model\Token|null
   *   The token object or NULL if not found.
   */
  public function getToken(string $token): ?Token {
    $query = $this->connection
      ->select(Token::TABLE_NAME, 't')
      ->fields('t')
      ->condition('token', $token);

    if ($this->configFactory->get('access_token_auth.settings')->get('mode') === TOKEN::DEFAULT_MODE) {
      $query->condition('expire_at', time(), '>');
    }
    else {
      $query->condition('used_status', FALSE);
    }

    $token_raw = $query->execute()->fetchObject();

    if (empty($token_raw)) {
      return NULL;
    }

    return new Token($token_raw->token, $token_raw->user_id, $token_raw->expire_at, $token_raw->used_status);
  }

  /**
   * Get a valid token from a specific user from the database.
   *
   * Used to not generate a new token if there's a valid one already.
   *
   * @param string $user_id
   *   The user ID to get valid tokens from.
   *
   * @return \Drupal\access_token_auth\Model\Token|null
   *   The token object or NULL if not found.
   */
  public function getValidUserToken(int $user_id): ?Token {
    $query = $this->connection
      ->select(Token::TABLE_NAME, 't')
      ->fields('t')
      ->condition('user_id', $user_id);

    if ($this->configFactory->get('access_token_auth.settings')->get('mode') === TOKEN::DEFAULT_MODE) {
      $query->condition('expire_at', time(), '>');
    }
    else {
      $query->condition('used_status', FALSE);
    }

    $token_raw = $query->execute()->fetchObject();

    if (empty($token_raw)) {
      return NULL;
    }

    return new Token($token_raw->token, $token_raw->user_id, $token_raw->expire_at, $token_raw->used_status);
  }

  /**
   * Invalidates a token once it's used.
   *
   * @param string $token
   *   The token value.
   */
  public function invalidateToken(string $token): void {
    $this->connection->update(Token::TABLE_NAME)
      ->fields([
        'used_status' => TRUE,
      ])
      ->condition('token', $token)
      ->execute();
  }

  /**
   * Remove the token from the database.
   *
   * @param string $token
   *   The token value.
   */
  public function removeByToken(string $token): void {
    $this->connection->delete(Token::TABLE_NAME)
      ->condition('token', $token)
      ->execute();
  }

  /**
   * Remove tokens by user id.
   *
   * @param int $user_id
   *   The user id the tokens belong to.
   */
  public function removeByUser(int $user_id): void {
    $this->connection->delete(Token::TABLE_NAME)
      ->condition('user_id', $user_id)
      ->execute();
  }

  /**
   * Remoce expired tokens.
   *
   * @param int $timestamp
   *   The date used as reference to remove older tokens.
   */
  public function removeExpiredTokens(int $timestamp): void {
    if ($this->configFactory->get('access_token_auth.settings')->get('mode') === TOKEN::DEFAULT_MODE) {
      $this->connection->delete(Token::TABLE_NAME)
        ->condition('expire_at', $timestamp, '<')
        ->execute();
    }
    else {
      $this->connection->delete(Token::TABLE_NAME)
        ->condition('used_status', TRUE)
        ->execute();
    }
  }

}
